@@ -25,47 +25,49 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from fido2.ctap import CtapError
+import csv as _csv
+import io
+import logging
+from time import sleep
+from typing import Optional, Sequence
+
+import click
+from fido2.ctap import STATUS, CtapError
 from fido2.ctap1 import ApduError
 from fido2.ctap2 import (
-    Ctap2,
-    ClientPin,
-    CredentialManagement,
-    FPBioEnrollment,
     CaptureError,
+    ClientPin,
+    Config,
+    CredentialManagement,
+    Ctap2,
+    FPBioEnrollment,
 )
-from fido2.pcsc import CtapPcscDevice
-from yubikit.core.fido import FidoConnection
-from yubikit.core.smartcard import SW
-from time import sleep
+from smartcard.Exceptions import CardConnectionException, NoCardException
+
+from yubikit.core import TRANSPORT
+from yubikit.core.fido import FidoConnection, SmartCardCtapDevice
+from yubikit.core.smartcard import SW, SmartCardConnection
+from yubikit.management import CAPABILITY
+
+from ..fido import fips_change_pin, fips_reset, fips_verify_pin, is_in_fips_mode
+from ..hid import list_ctap_devices
+from ..pcsc import ScardYubiKeyDevice
 from .util import (
-    click_postpone_execution,
-    click_prompt,
+    CliFail,
     click_force_option,
     click_group,
-    prompt_timeout,
+    click_postpone_execution,
+    click_prompt,
     is_yk4_fips,
+    pretty_print,
+    prompt_for_touch,
+    prompt_timeout,
 )
-from .util import CliFail
-from ..fido import is_in_fips_mode, fips_reset, fips_change_pin, fips_verify_pin
-from ..hid import list_ctap_devices
-from ..pcsc import list_devices as list_ccid
-from smartcard.Exceptions import NoCardException, CardConnectionException
-from typing import Optional, Sequence, List
-
-import io
-import csv as _csv
-import click
-import logging
 
 logger = logging.getLogger(__name__)
 
 
-FIPS_PIN_MIN_LENGTH = 6
-PIN_MIN_LENGTH = 4
-
-
-@click_group(connections=[FidoConnection])
+@click_group(connections=[FidoConnection, SmartCardConnection])
 @click.pass_context
 @click_postpone_execution
 def fido(ctx):
@@ -84,7 +86,14 @@ def fido(ctx):
 
     """
     dev = ctx.obj["device"]
-    conn = dev.open_connection(FidoConnection)
+    resolve_scp = ctx.obj.get("scp")
+    if resolve_scp:
+        s_conn = dev.open_connection(SmartCardConnection)
+        scp_params = resolve_scp(s_conn)
+        conn = SmartCardCtapDevice(s_conn, scp_params)
+    else:
+        conn = dev.open_connection(FidoConnection)
+
     ctx.call_on_close(conn.close)
     ctx.obj["conn"] = conn
     try:
@@ -99,56 +108,66 @@ def info(ctx):
     """
     Display general status of the FIDO2 application.
     """
-    conn = ctx.obj["conn"]
+    info = ctx.obj["info"]
     ctap2 = ctx.obj.get("ctap2")
 
-    if is_yk4_fips(ctx.obj["info"]):
-        click.echo("FIPS Approved Mode: " + ("Yes" if is_in_fips_mode(conn) else "No"))
-    elif ctap2:
+    data: dict = {}
+    lines: list = [data]
+
+    if CAPABILITY.FIDO2 in info.fips_capable:
+        data["FIPS approved"] = CAPABILITY.FIDO2 in info.fips_approved
+    elif is_yk4_fips(info):
+        data["FIPS approved"] = is_in_fips_mode(ctx.obj["conn"])
+
+    if ctap2:
+        if ctap2.info.aaguid:
+            data["AAGUID"] = str(ctap2.info.aaguid)
         client_pin = ClientPin(ctap2)  # N.B. All YubiKeys with CTAP2 support PIN.
         if ctap2.info.options["clientPin"]:
             if ctap2.info.force_pin_change:
-                click.echo(
-                    "NOTE: The FIDO PID is disabled and must be changed before it can "
+                lines.append(
+                    "NOTE: The FIDO PIN is disabled and must be changed before it can "
                     "be used!"
                 )
             pin_retries, power_cycle = client_pin.get_pin_retries()
             if pin_retries:
-                click.echo(f"PIN is set, with {pin_retries} attempt(s) remaining.")
+                data["PIN"] = f"{pin_retries} attempt(s) remaining"
                 if power_cycle:
-                    click.echo(
+                    lines.append(
                         "PIN is temporarily blocked. "
                         "Remove and re-insert the YubiKey to unblock."
                     )
             else:
-                click.echo("PIN is set, but has been blocked.")
+                data["PIN"] = "Blocked"
         else:
-            click.echo("PIN is not set.")
+            data["PIN"] = "Not set"
+        data["Minimum PIN length"] = ctap2.info.min_pin_length
 
         bio_enroll = ctap2.info.options.get("bioEnroll")
         if bio_enroll:
             uv_retries = client_pin.get_uv_retries()
             if uv_retries:
-                click.echo(
-                    f"Fingerprints registered, with {uv_retries} attempt(s) "
-                    "remaining."
-                )
+                data["Fingerprints"] = f"Registered, {uv_retries} attempt(s) remaining"
             else:
-                click.echo(
-                    "Fingerprints registered, but blocked until PIN is verified."
-                )
+                data["Fingerprints"] = "Registered, blocked until PIN is verified"
         elif bio_enroll is False:
-            click.echo("No fingerprints have been registered.")
+            data["Fingerprints"] = "Not registered"
 
         always_uv = ctap2.info.options.get("alwaysUv")
         if always_uv is not None:
-            click.echo(
-                "Always Require User Verification is turned "
-                + ("on." if always_uv else "off.")
-            )
+            data["Always Require UV"] = "On" if always_uv else "Off"
 
+        remaining_creds = ctap2.info.remaining_disc_creds
+        if remaining_creds is not None:
+            data["Credential storage remaining"] = remaining_creds
+
+        ep = ctap2.info.options.get("ep")
+        if ep is not None:
+            data["Enterprise Attestation"] = "Enabled" if ep else "Disabled"
     else:
-        click.echo("PIN is not supported.")
+        data["PIN"] = "Not supported"
+
+    click.echo("\n".join(pretty_print(lines)))
 
 
 @fido.command("reset")
@@ -165,16 +184,20 @@ def reset(ctx, force):
     inserted, and requires a touch on the YubiKey.
     """
 
-    conn = ctx.obj["conn"]
+    info = ctx.obj["info"]
+    if CAPABILITY.FIDO2 in info.reset_blocked:
+        raise CliFail(
+            "Cannot perform FIDO reset when PIV is configured, "
+            "use 'ykman config reset' for full factory reset."
+        )
 
-    if isinstance(conn, CtapPcscDevice):  # NFC
-        readers = list_ccid(conn._name)
-        if not readers or readers[0].reader.name != conn._name:
-            raise CliFail("Unable to isolate NFC reader.")
-        dev = readers[0]
-        logger.debug(f"use: {dev}")
+    dev = ctx.obj["device"]
+    conn = ctx.obj["conn"]
+    if isinstance(dev, ScardYubiKeyDevice):  # NFC
         is_fips = False
 
+        conn.close()
+
         def prompt_re_insert():
             click.echo(
                 "Remove and re-place your YubiKey on the NFC reader to perform the "
@@ -226,10 +249,10 @@ def reset(ctx, force):
         )
         if is_fips:
             destroy_input = click_prompt(
-                "WARNING! This is a YubiKey FIPS device. This command will also "
-                "overwrite the U2F attestation key; this action cannot be undone and "
-                "this YubiKey will no longer be a FIPS compliant device.\n"
-                'To proceed, please enter the text "OVERWRITE"',
+                "WARNING! This is a YubiKey FIPS (4 Series) device. This command will "
+                "also overwrite the U2F attestation key; this action cannot be undone "
+                "and this YubiKey will no longer be a FIPS compliant device.\n"
+                'To proceed, enter the text "OVERWRITE"',
                 default="",
                 show_default=False,
             )
@@ -239,12 +262,17 @@ def reset(ctx, force):
         conn = prompt_re_insert()
 
     try:
-        with prompt_timeout():
-            if is_fips:
+        if is_fips:
+            with prompt_timeout():
                 fips_reset(conn)
-            else:
-                Ctap2(conn).reset()
-        logger.info("FIDO application data reset")
+        else:
+
+            def on_keepalive(status):
+                if status == STATUS.UPNEEDED:
+                    prompt_for_touch()
+
+            Ctap2(conn).reset(on_keepalive=on_keepalive)
+        click.echo("FIDO application data reset.")
     except CtapError as e:
         if e.code == CtapError.ERR.ACTION_TIMEOUT:
             raise CliFail(
@@ -256,7 +284,7 @@ def reset(ctx, force):
                 "YubiKey is inserted."
             )
         else:
-            raise CliFail(f"Reset failed: {e.code.name}")
+            raise CliFail(f"Reset failed: {e.code.name}.")
     except ApduError as e:  # From fips_reset
         if e.code == SW.COMMAND_NOT_ALLOWED:
             raise CliFail(
@@ -274,8 +302,7 @@ def _fail_pin_error(ctx, e, other="%s"):
         raise CliFail("Wrong PIN.")
     elif e.code == CtapError.ERR.PIN_AUTH_BLOCKED:
         raise CliFail(
-            "PIN authentication is currently blocked. "
-            "Remove and re-insert the YubiKey."
+            "PIN authentication is currently blocked. Remove and re-insert the YubiKey."
         )
     elif e.code == CtapError.ERR.PIN_BLOCKED:
         raise CliFail("PIN is blocked.")
@@ -298,39 +325,57 @@ def access():
     "-u",
     "--u2f",
     is_flag=True,
-    help="set FIDO U2F PIN instead of FIDO2 PIN (YubiKey 4 FIPS only)",
+    help="set FIDO U2F PIN instead of FIDO2 PIN (YubiKey FIPS only)",
 )
 def change_pin(ctx, pin, new_pin, u2f):
     """
     Set or change the PIN code.
 
-    The FIDO2 PIN must be at least 4 characters long, and supports any type
-    of alphanumeric characters.
+    The FIDO2 PIN must be at least 4 characters long, and supports any type of
+    alphanumeric characters. Some YubiKeys can be configured to require a longer
+    PIN.
 
-    On YubiKey FIPS, a PIN can be set for FIDO U2F. That PIN must be at least
+    On YubiKey FIPS (4 Series), a PIN can be set for FIDO U2F. That PIN must be at least
     6 characters long.
     """
 
-    is_fips = is_yk4_fips(ctx.obj["info"])
+    info = ctx.obj["info"]
+    is_fips = is_yk4_fips(info)
 
     if is_fips and not u2f:
         raise CliFail(
-            "This is a YubiKey FIPS. To set the U2F PIN, pass the --u2f option."
+            "This is a YubiKey FIPS (4 Series). "
+            "To set the U2F PIN, pass the --u2f option."
         )
 
     if u2f and not is_fips:
         raise CliFail(
-            "This is not a YubiKey 4 FIPS, and therefore does not support a U2F PIN. "
-            "To set the FIDO2 PIN, remove the --u2f option."
+            "This is not a YubiKey FIPS (4 Series), and therefore does not support a "
+            "U2F PIN. To set the FIDO2 PIN, remove the --u2f option."
         )
 
     if is_fips:
         conn = ctx.obj["conn"]
+        min_len = 6
     else:
         ctap2 = ctx.obj.get("ctap2")
         if not ctap2:
             raise CliFail("PIN is not supported on this YubiKey.")
         client_pin = ClientPin(ctap2)
+        min_len = ctap2.info.min_pin_length
+    if (
+        info._is_bio
+        and CAPABILITY.PIV in info.config.enabled_capabilities[TRANSPORT.USB]
+    ):
+        max_len = 8
+    else:
+        max_len = 63
+
+    def _fail_if_not_valid_pin(pin=None, name="PIN"):
+        if not pin or len(pin) < min_len:
+            raise CliFail(f"{name} must be at least {min_len} characters long.")
+        if len(pin) > max_len:
+            raise CliFail(f"{name} must be at most {max_len} characters long.")
 
     def prompt_new_pin():
         return click_prompt(
@@ -340,8 +385,6 @@ def change_pin(ctx, pin, new_pin, u2f):
         )
 
     def change_pin(pin, new_pin):
-        if pin is not None:
-            _fail_if_not_valid_pin(ctx, pin, is_fips)
         try:
             if is_fips:
                 try:
@@ -350,7 +393,7 @@ def change_pin(ctx, pin, new_pin, u2f):
                 except ApduError as e:
                     if e.code == SW.WRONG_LENGTH:
                         pin = _prompt_current_pin()
-                        _fail_if_not_valid_pin(ctx, pin, is_fips)
+                        _fail_if_not_valid_pin(pin)
                         fips_change_pin(conn, pin, new_pin)
                     else:
                         raise
@@ -360,9 +403,9 @@ def change_pin(ctx, pin, new_pin, u2f):
 
         except CtapError as e:
             if e.code == CtapError.ERR.PIN_POLICY_VIOLATION:
-                raise CliFail("New PIN doesn't meet policy requirements.")
+                raise CliFail("New PIN doesn't meet complexity requirements.")
             else:
-                _fail_pin_error(ctx, e, "Failed to change PIN: %s")
+                _fail_pin_error(ctx, e, "Failed to change PIN: %s.")
 
         except ApduError as e:
             if e.code == SW.VERIFY_FAIL_NO_RETRY:
@@ -370,17 +413,16 @@ def change_pin(ctx, pin, new_pin, u2f):
             elif e.code == SW.AUTH_METHOD_BLOCKED:
                 raise CliFail("PIN is blocked.")
             else:
-                raise CliFail(f"Failed to change PIN: SW={e.code:04x}")
+                raise CliFail(f"Failed to change PIN: SW={e.code:04x}.")
 
     def set_pin(new_pin):
-        _fail_if_not_valid_pin(ctx, new_pin, is_fips)
         try:
             client_pin.set_pin(new_pin)
         except CtapError as e:
             if e.code == CtapError.ERR.PIN_POLICY_VIOLATION:
-                raise CliFail("PIN is too long.")
+                raise CliFail("New PIN doesn't meet complexity requirements.")
             else:
-                raise CliFail(f"Failed to set PIN: {e.code}")
+                raise CliFail(f"Failed to set PIN: {e.code}.")
 
     if not is_fips:
         if ctap2.info.options.get("clientPin"):
@@ -392,18 +434,16 @@ def change_pin(ctx, pin, new_pin, u2f):
 
     if not new_pin:
         new_pin = prompt_new_pin()
+    _fail_if_not_valid_pin(new_pin, "New PIN")
 
     if is_fips:
-        _fail_if_not_valid_pin(ctx, new_pin, is_fips)
         change_pin(pin, new_pin)
     else:
-        if len(new_pin) < ctap2.info.min_pin_length:
-            raise CliFail("New PIN is too short.")
         if ctap2.info.options.get("clientPin"):
             change_pin(pin, new_pin)
         else:
             set_pin(new_pin)
-    logger.info("FIDO PIN updated")
+    click.echo("FIDO PIN updated.")
 
 
 def _require_pin(ctx, pin, feature="This feature"):
@@ -427,7 +467,7 @@ def verify(ctx, pin):
     Verify the FIDO PIN against a YubiKey.
 
     For YubiKeys supporting FIDO2 this will reset the "retries" counter of the PIN.
-    For YubiKey FIPS this will unlock the session, allowing U2F registration.
+    For YubiKey FIPS (4 Series) this will unlock the session, allowing U2F registration.
     """
 
     ctap2 = ctx.obj.get("ctap2")
@@ -440,9 +480,8 @@ def verify(ctx, pin):
                 pin, ClientPin.PERMISSION.GET_ASSERTION, "ykman.example.com"
             )
         except CtapError as e:
-            raise CliFail(f"PIN verification failed: {e}")
+            raise CliFail(f"PIN verification failed: {e}.")
     elif is_yk4_fips(ctx.obj["info"]):
-        _fail_if_not_valid_pin(ctx, pin, True)
         try:
             fips_verify_pin(ctx.obj["conn"], pin)
         except ApduError as e:
@@ -453,22 +492,100 @@ def verify(ctx, pin):
             elif e.code == SW.COMMAND_NOT_ALLOWED:
                 raise CliFail("PIN is not set.")
             else:
-                raise CliFail(f"PIN verification failed: {e.code.name}")
+                raise CliFail(f"PIN verification failed: {e.code.name}.")
     else:
         raise CliFail("This YubiKey does not support a FIDO PIN.")
     click.echo("PIN verified.")
 
 
+def _init_config(ctx, pin):
+    ctap2 = ctx.obj.get("ctap2")
+    if not Config.is_supported(ctap2.info):
+        raise CliFail("Authenticator Configuration is not supported on this YubiKey.")
+
+    protocol = None
+    token = None
+    if ctap2.info.options.get("clientPin"):
+        pin = _require_pin(ctx, pin, "Authenticator Configuration")
+        client_pin = ClientPin(ctap2)
+        try:
+            protocol = client_pin.protocol
+            token = client_pin.get_pin_token(
+                pin, ClientPin.PERMISSION.AUTHENTICATOR_CFG
+            )
+        except CtapError as e:
+            _fail_pin_error(ctx, e, "PIN error: %s.")
+
+    return Config(ctap2, protocol, token)
+
+
+@access.command("force-change")
+@click.pass_context
+@click.option("-P", "--pin", help="PIN code")
+def force_pin_change(ctx, pin):
+    """
+    Force the PIN to be changed to a new value before use.
+    """
+    options = ctx.obj["ctap2"].info.options if "ctap2" in ctx.obj else None
+    if options is None or not options.get("setMinPINLength"):
+        raise CliFail("Force change PIN is not supported on this YubiKey.")
+    if not options.get("clientPin"):
+        raise CliFail("No PIN is set.")
+
+    config = _init_config(ctx, pin)
+    config.set_min_pin_length(force_change_pin=True)
+    click.echo("Force PIN change set.")
+
+
+@access.command("set-min-length")
+@click.pass_context
+@click.option("-P", "--pin", help="PIN code")
+@click.option("-R", "--rp-id", multiple=True, help="RP ID to allow")
+@click.argument("length", type=click.IntRange(4, 63))
+def set_min_pin_length(ctx, pin, rp_id, length):
+    """
+    Set the minimum length allowed for PIN.
+
+    Optionally use the --rp-id option to specify which RPs are allowed to request this
+    information.
+    """
+    info = ctx.obj["ctap2"].info if "ctap2" in ctx.obj else None
+    if info is None or not info.options.get("setMinPINLength"):
+        raise CliFail("Set minimum PIN length is not supported on this YubiKey.")
+    if info.options.get("alwaysUv") and not info.options.get("clientPin"):
+        raise CliFail(
+            "Setting min PIN length requires a PIN to be set when alwaysUv is enabled."
+        )
+
+    min_len = info.min_pin_length
+    if length < min_len:
+        raise CliFail(f"Cannot set a minimum length that is shorter than {min_len}.")
+
+    dev_info = ctx.obj["info"]
+    if (
+        dev_info._is_bio
+        and CAPABILITY.PIV in dev_info.config.enabled_capabilities[TRANSPORT.USB]
+        and length > 8
+    ):
+        raise CliFail("Cannot set a minimum length that is longer than 8.")
+
+    config = _init_config(ctx, pin)
+    if rp_id:
+        ctap2 = ctx.obj.get("ctap2")
+        cap = ctap2.info.max_rpids_for_min_pin
+        if len(rp_id) > cap:
+            raise CliFail(
+                f"Authenticator supports up to {cap} RP IDs ({len(rp_id)} given)."
+            )
+
+    config.set_min_pin_length(min_pin_length=length, rp_ids=rp_id)
+    click.echo("Minimum PIN length set.")
+
+
 def _prompt_current_pin(prompt="Enter your current PIN"):
     return click_prompt(prompt, hide_input=True)
 
 
-def _fail_if_not_valid_pin(ctx, pin=None, is_fips=False):
-    min_length = FIPS_PIN_MIN_LENGTH if is_fips else PIN_MIN_LENGTH
-    if not pin or len(pin) < min_length:
-        ctx.fail(f"PIN must be over {min_length} characters long")
-
-
 def _gen_creds(credman):
     data = credman.get_metadata()
     if data.get(CredentialManagement.RESULT.EXISTING_CRED_COUNT) == 0:
@@ -484,7 +601,7 @@ def _gen_creds(credman):
             )
 
 
-def _format_table(headings: Sequence[str], rows: List[Sequence[str]]) -> str:
+def _format_table(headings: Sequence[str], rows: list[Sequence[str]]) -> str:
     all_rows = [headings] + rows
     padded_rows = [["" for cell in row] for row in all_rows]
 
@@ -533,7 +650,7 @@ def _init_credman(ctx, pin):
     try:
         token = client_pin.get_pin_token(pin, ClientPin.PERMISSION.CREDENTIAL_MGMT)
     except CtapError as e:
-        _fail_pin_error(ctx, e, "PIN error: %s")
+        _fail_pin_error(ctx, e, "PIN error: %s.")
 
     return CredentialManagement(ctap2, client_pin.protocol, token)
 
@@ -615,7 +732,7 @@ def creds_delete(ctx, credential_id, pin, force):
         ):
             try:
                 credman.delete_cred(cred_id)
-                logger.info("Credential deleted")
+                click.echo("Credential deleted.")
             except CtapError:
                 raise CliFail("Failed to delete credential.")
     else:
@@ -658,7 +775,7 @@ def _init_bio(ctx, pin):
     try:
         token = client_pin.get_pin_token(pin, ClientPin.PERMISSION.BIO_ENROLL)
     except CtapError as e:
-        _fail_pin_error(ctx, e, "PIN error: %s")
+        _fail_pin_error(ctx, e, "PIN error: %s.")
 
     return FPBioEnrollment(ctap2, client_pin.protocol, token)
 
@@ -717,7 +834,7 @@ def bio_enroll(ctx, name, pin):
                 )
             elif e.code == CtapError.ERR.USER_ACTION_TIMEOUT:
                 raise CliFail("Failed to add fingerprint due to user inactivity.")
-            raise CliFail(f"Failed to add fingerprint: {e.code.name}")
+            raise CliFail(f"Failed to add fingerprint: {e.code.name}.")
     logger.info("Fingerprint template registered")
     click.echo("Capture complete.")
     bio.set_name(template_id, name)
@@ -748,7 +865,7 @@ def bio_rename(ctx, template_id, name, pin):
         raise CliFail(f"No fingerprint matching ID={template_id}.")
 
     bio.set_name(key, name)
-    logger.info("Fingerprint template renamed")
+    click.echo("Fingerprint template renamed.")
 
 
 @bio.command("delete")
@@ -775,7 +892,7 @@ def bio_delete(ctx, template_id, pin, force):
         # Match using template_id as NAME
         matches = [k for k in enrollments if enrollments[k] == template_id]
         if len(matches) == 0:
-            raise CliFail(f"No fingerprint matching ID={template_id}")
+            raise CliFail(f"No fingerprint matching ID={template_id}.")
         elif len(matches) > 1:
             raise CliFail(
                 f"Multiple matches for NAME={template_id}. "
@@ -787,6 +904,56 @@ def bio_delete(ctx, template_id, pin, force):
     if force or click.confirm(f"Delete fingerprint {_format_fp(key, name)}?"):
         try:
             bio.remove_enrollment(key)
-            logger.info("Fingerprint template deleted")
+            click.echo("Fingerprint template deleted.")
         except CtapError as e:
-            raise CliFail(f"Failed to delete fingerprint: {e.code.name}")
+            raise CliFail(f"Failed to delete fingerprint: {e.code.name}.")
+
+
+@fido.group("config")
+def config():
+    """
+    Manage FIDO configuration.
+    """
+
+
+@config.command("toggle-always-uv")
+@click.pass_context
+@click.option("-P", "--pin", help="PIN code")
+def toggle_always_uv(ctx, pin):
+    """
+    Toggles the state of Always Require User Verification.
+    """
+    options = ctx.obj.get("ctap2").info.options if "ctap2" in ctx.obj else None
+    if not options or "alwaysUv" not in options:
+        raise CliFail("Always Require UV is not supported on this YubiKey.")
+
+    info = ctx.obj["info"]
+    if CAPABILITY.FIDO2 in info.fips_capable:
+        raise CliFail("Always Require UV can not be disabled on this YubiKey.")
+
+    always_uv = options["alwaysUv"]
+
+    config = _init_config(ctx, pin)
+    config.toggle_always_uv()
+    click.echo(f"Always Require UV is {'off' if always_uv else 'on'}.")
+
+
+@config.command("enable-ep-attestation")
+@click.pass_context
+@click.option("-P", "--pin", help="PIN code")
+def enable_ep_attestation(ctx, pin):
+    """
+    Enables Enterprise Attestation for Authenticators pre-configured to support it.
+    """
+    options = ctx.obj.get("ctap2").info.options if "ctap2" in ctx.obj else None
+    if not options or "ep" not in options:
+        raise CliFail("Enterprise Attestation is not supported on this YubiKey.")
+    if options.get("alwaysUv") and not options.get("clientPin"):
+        raise CliFail(
+            "Enabling Enterprise Attestation requires a PIN to be set when alwaysUv is "
+            "enabled."
+        )
+
+    config = _init_config(ctx, pin)
+    config.enable_enterprise_attestation()
+    click.echo("Enterprise Attestation enabled.")
